import uuid

from django.db import models
from django.contrib.contenttypes.models import ContentType
from django_resaas.core.base.models import TimeModel


class EntidadeGroup(TimeModel):
    entidade = models.ForeignKey('django_resaas.Entidade', on_delete=models.CASCADE)
    group = models.ForeignKey('auth.Group', on_delete=models.CASCADE)
    
    class Meta:
        unique_together = ("entidade", "group")
        permissions = ()

    def __str__(self):
        return f'{self.entidade.nome} | {self.group.name}'





from rest_framework import filters
from rest_framework import viewsets


from django_resaas.models.idioma import Idioma
from django_resaas.data.idioma.serializers.idioma import IdiomaSerializer


class TraducaoAPIView(viewsets.ModelViewSet):
    filter_backends = (filters.SearchFilter,)
    serializer_class = IdiomaSerializer
    queryset = Idioma.objects.all()

    def get_queryset(self):
        return self.queryset.filter()


# =========================
# Python standard library
# =========================
import json


# =========================
# Django
# =========================
from django.contrib.auth.models import Group
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.db.models import F
from django.http import Http404


# =========================
# Django REST Framework
# =========================
from rest_framework import filters
from rest_framework import status
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response


# =========================
# Local application (absolute import)
# =========================
from django_resaas.data.group.serializers.grupo import GrupoSerializer


class GrupoAPIView(viewsets.ModelViewSet):
    """
    API de gestão de Grupos (Profiles / Roles).
    """

    serializer_class = GrupoSerializer
    queryset = Group.objects.all()
    lookup_field = "id"
    filter_backends = (filters.SearchFilter,)
    search_fields = ["id", "name"]
    pagination_class = None

    # -------------------------
    # Queryset
    # -------------------------

    def get_queryset(self):
        # Group NÃO tem codename
        return self.queryset.order_by("name")

    # -------------------------
    # Retrieve
    # -------------------------

    def retrieve(self, request, id, *args, **kwargs):
        grupo = self.get_object()

        # Se ?permissions=1 → retorna permissões do grupo
        if request.query_params.get("permissions"):
            permissions = (
                grupo.permissions
                .annotate(
                    content_type_model=F("content_type__model"),
                    content_type_app=F("content_type__app_label"),
                )
                .order_by("content_type_app", "content_type_model", "codename")
            )

            return Response(
                [
                    {
                        "id": p.id,
                        "name": p.name,
                        "codename": p.codename,
                        "content_type": p.content_type.id,
                        "content_type_model": p.content_type_model,
                        "content_type_app": p.content_type_app,
                    }
                    for p in permissions
                ],
                status=status.HTTP_200_OK,
            )

        serializer = self.get_serializer(grupo)
        return Response(serializer.data, status=status.HTTP_200_OK)

    # -------------------------
    # Update
    # -------------------------

    def update(self, request, id, *args, **kwargs):
        grupo = self.get_object()
        grupo.name = request.data.get("name", grupo.name)
        grupo.save()

        return Response(
            {
                "id": grupo.id,
                "name": grupo.name,
                "alert_success": f'%-{grupo.name}-% foi actualizado com sucesso',
            },
            status=status.HTTP_202_ACCEPTED,
        )

    # -------------------------
    # Destroy
    # -------------------------

    def destroy(self, request, id, *args, **kwargs):
        grupo = self.get_object()
        nome = grupo.name
        grupo.delete()

        return Response(
            {
                "alert_success": f"<b>{nome}</b> foi apagado com sucesso"
            },
            status=status.HTTP_202_ACCEPTED,
        )

    # -------------------------
    # Actions
    # -------------------------

    @action(detail=True, methods=["POST"])
    def addPermission(self, request, id):
        grupo = self.get_object()

        codename = request.data.get("codename")
        name = request.data.get("name")

        if not codename or not name:
            return Response(
                {"alert_error": "codename e name são obrigatórios"},
                status=status.HTTP_400_BAD_REQUEST,
            )

        content_type, _ = ContentType.objects.get_or_create(
            app_label="custom",
            model="custom_permission",
        )

        permission, _ = Permission.objects.get_or_create(
            content_type=content_type,
            codename=codename,
            defaults={"name": name},
        )

        grupo.permissions.add(permission)

        return Response(
            {
                "id": permission.id,
                "codename": permission.codename,
                "name": permission.name,
                "alert_success": f'Permissão <b>{permission.name}</b> adicionada com sucesso',
            },
            status=status.HTTP_201_CREATED,
        )

    



from django.db.models import Q

from rest_framework import serializers
from rest_framework.exceptions import AuthenticationFailed

from django_resaas.models.user import User
from django_resaas.core.utils.translate import Translate
from django.utils import timezone


def authenticate(value=None, password=None):
    try:
        user = User.objects.get(
            Q(email=value) |
            Q(username=value) |
            Q(mobile=value)
        )
    except User.DoesNotExist:
        return None

    if user.check_password(password):
        user.last_login = timezone.now()
        user.save(update_fields=['last_login'])
        return user

    return None

class LoginSerializer(serializers.Serializer):
    identifier = serializers.CharField(write_only=True)
    password = serializers.CharField(write_only=True)

    id = serializers.UUIDField(read_only=True)
    email = serializers.EmailField(read_only=True)
    username = serializers.CharField(read_only=True)
    mobile = serializers.CharField(read_only=True, allow_null=True)
    tokens = serializers.SerializerMethodField(read_only=True)

    def get_tokens(self, obj):
        return obj["tokens"]

    def validate(self, attrs):
        request = self.context.get("request")

        identifier = attrs.get("identifier", "").strip()
        password = attrs.get("password")

        user = authenticate(
            value=identifier,
            password=password,
        )

        if not user:
            raise AuthenticationFailed(
                Translate.tdc(request, "Credenciais inválidas")
            )

        if not user.is_active:
            raise AuthenticationFailed(
                Translate.tdc(request, "Conta desactivada")
            )

        if not user.is_verified_email:
            raise AuthenticationFailed(
                Translate.tdc(request, "Email não verificado")
            )

        return {
            "id": user.id,
            "email": user.email,
            "username": user.username,
            "mobile": user.mobile,
            "tokens": user.tokens(),
        }



from rest_framework.response import Response
from django_resaas.core.utils.translate import Translate
from django_resaas.core.utils.clean import clean_name

class ApiResponse:

    @staticmethod
    def _msg(request, key):
        key = clean_name(key)
        return key if " " in key else Translate.tdc(request, key)

    @classmethod
    def ok(cls, request, key, status=200, **extra):
        return Response({
            "alert_success": cls._msg(request, key),
            **extra
        }, status=status)

    @classmethod
    def all(cls, request, status=200, **extra):
        return Response({
            **extra
        }, status=status)

    @classmethod
    def fail(cls, request, key, status=400, **extra):
        return Response({
            "alert_error": cls._msg(request, key),
            **extra
        }, status=status)

    @classmethod
    def warn(cls, request, key, status=200, **extra):
        return Response({
            "alert_warning": cls._msg(request, key),
            **extra
        }, status=status)

# ==============================
# Shortcuts (DX sugar 🔥)
# ==============================

def ok(request, key, **extra):
    return ApiResponse.ok(request, key, **extra)

def all(request, **extra):
    return ApiResponse.all(request, **extra)


def fail(request, key, **extra):
    return ApiResponse.fail(request, key, **extra)


def warn(request, key, **extra):
    return ApiResponse.warn(request, key, **extra)



# Generated by Django 5.2.10 on 2026-03-07 15:57

import django.db.models.deletion
import uuid
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_resaas', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='LayoutSetting',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('deleted_at', models.DateTimeField(blank=True, null=True)),
                ('estado', models.IntegerField(choices=[(0, 'Inativo'), (1, 'Ativo')], default=0, null=True)),
                ('primary', models.CharField(blank=True, default='', max_length=50, null=True)),
                ('secondary', models.CharField(blank=True, default='', max_length=50, null=True)),
                ('accent', models.CharField(blank=True, default='', max_length=50, null=True)),
                ('dark', models.CharField(blank=True, default='', max_length=50, null=True)),
                ('positive', models.CharField(blank=True, default='', max_length=50, null=True)),
                ('negative', models.CharField(blank=True, default='', max_length=50, null=True)),
                ('info', models.CharField(blank=True, default='', max_length=50, null=True)),
                ('warning', models.CharField(blank=True, default='', max_length=50, null=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='%(class)s_created', to=settings.AUTH_USER_MODEL)),
                ('updated_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='%(class)s_updated', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'LayoutSetting',
                'verbose_name_plural': 'LayoutSettings',
                'permissions': (),
            },
        ),
        migrations.AddField(
            model_name='entidade',
            name='layout_settings',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='django_resaas.layoutsetting'),
        ),
        migrations.AddField(
            model_name='tipoentidade',
            name='layout_settings',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='django_resaas.layoutsetting'),
        ),
        migrations.CreateModel(
            name='Theme',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('deleted_at', models.DateTimeField(blank=True, null=True)),
                ('estado', models.IntegerField(choices=[(0, 'Inativo'), (1, 'Ativo')], default=0, null=True)),
                ('nome', models.CharField(max_length=100)),
                ('primary', models.CharField(default='#1976D2', max_length=7)),
                ('secondary', models.CharField(default='#26A69A', max_length=7)),
                ('accent', models.CharField(default='#9C27B0', max_length=7)),
                ('positive', models.CharField(default='#21BA45', max_length=7)),
                ('negative', models.CharField(default='#C10015', max_length=7)),
                ('warning', models.CharField(default='#F2C037', max_length=7)),
                ('info', models.CharField(default='#31CCEC', max_length=7)),
                ('dark', models.CharField(default='#1d1d1d', max_length=7)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='%(class)s_created', to=settings.AUTH_USER_MODEL)),
                ('updated_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='%(class)s_updated', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Theme',
                'verbose_name_plural': 'Theme',
                'permissions': (),
            },
        ),
        migrations.AddField(
            model_name='entidade',
            name='theme',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='django_resaas.theme'),
        ),
        migrations.AddField(
            model_name='tipoentidade',
            name='theme',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='django_resaas.theme'),
        ),
    ]



class DiskManegarService:
    def __init__(self, request):
        self.request = request
        self.entidade_id = request.entidade_id
        self.sucursal_id = request.sucursal_id
        self.user = request.user